@@ -1,41 +1,41 @@
 #! /usr/local/bin/ruby -w
 require 'RMagick'
-include Magick
 
 # Add a method for drawing braces.
-class Draw
+module Magick
+    class Draw
 
-    # (w,h) - width & height of rectangle enclosing brace.
-    # Normally the brace is drawn with its opening to the
-    # left and its lower point on the origin.
-    #
-    # Set w < 0 to draw right-opening brace. Set h < 0 to
-    # position top point at origin.
-    #
-    # The placement & orientation is affected by the
-    # current user coordinate system.
-    def brace(w, h)
-        raise(ArgumentError, "width must be != 0") unless w != 0
-        raise(ArgumentError, "height must be != 0") unless h != 0
-        path("M0,0 Q#{w},0 #{w/2.0},#{-h/4.0} T#{w},#{-h/2.0}" +
-             "Q0,#{-h/2.0} #{w/2.0},#{-(3.0*h/4.0)} T0,#{-h}")
-    end
-
-end     # class Draw
+        # (w,h) - width & height of rectangle enclosing brace.
+        # Normally the brace is drawn with its opening to the
+        # left and its lower point on the origin.
+        #
+        # Set w < 0 to draw right-opening brace. Set h < 0 to
+        # position top point at origin.
+        #
+        # The placement & orientation is affected by the
+        # current user coordinate system.
+        def brace(w, h)
+            raise(ArgumentError, "width must be != 0") unless w != 0
+            raise(ArgumentError, "height must be != 0") unless h != 0
+            path("M0,0 Q#{w},0 #{w/2.0},#{-h/4.0} T#{w},#{-h/2.0}" +
+                 "Q0,#{-h/2.0} #{w/2.0},#{-(3.0*h/4.0)} T0,#{-h}")
+        end
 
+    end     # class Draw
+end
 
 
 
 Origin_x = 110
 Origin_y = 230
 Glyph = 'g'
-Face = ARGV[0] ? ARGV[0] : "Times"
+Face = RUBY_PLATFORM =~ /mswin/ ? "Verdana" : "Times"
 
-canvas = Image.new(410,320,HatchFill.new('white', 'lightcyan2'))
+canvas = Magick::Image.new(410, 320, Magick::HatchFill.new('white', 'lightcyan2'))
 
 # Draw a big lowercase 'g' on the canvas. Leave room on all sides for
 # the labels. Use 'undercolor' to set off the glyph.
-glyph = Draw.new
+glyph = Magick::Draw.new
 glyph.annotate(canvas, 0, 0, Origin_x, Origin_y, Glyph) do
     glyph.pointsize = 124
     glyph.stroke = 'none'
@@ -47,7 +47,7 @@ end
 # Call get_type_metrics. This is what this example's all about.
 metrics = glyph.get_type_metrics(canvas, Glyph)
 
-gc = Draw.new
+gc = Magick::Draw.new
 gc.translate(Origin_x, Origin_y)
 
 # Draw the origin as a big red dot.
@@ -88,8 +88,8 @@ gc.draw(canvas)
 
 # Draw the braces and labels. Position the braces by transforming the
 # user coordinate system with translate and rotate methods.
-gc = Draw.new
-gc.font_family('Times')
+gc = Magick::Draw.new
+gc.font_family('Face')
 gc.pointsize(13)
 gc.fill('none')
 gc.stroke('black')
