@@ -1,6 +1,6 @@
-/* $Id: rmilist.c,v 1.41 2006/06/20 23:33:07 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.71 2008/05/21 22:32:40 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2006 by Timothy P. Hunter
+|                Copyright (C) 2008 by Timothy P. Hunter
 | Name:     rmilist.c
 | Author:   Tim Hunter
 | Purpose:  ImageList class method definitions for RMagick
@@ -8,6 +8,13 @@
 
 #include "rmagick.h"
 
+static Image *clone_imagelist(Image *);
+static Image *images_from_imagelist(VALUE);
+static long imagelist_length(VALUE);
+static VALUE imagelist_scene_eq(VALUE, VALUE);
+static void imagelist_push(VALUE, VALUE);
+static VALUE ImageList_new(void);
+
 
 /*
     Method:     ImageList#animate(<delay>)
@@ -23,13 +30,14 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     Info *info;
     volatile VALUE info_obj;
 
-    // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
-
     if (argc > 1)
     {
         rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
     }
+
+    // Convert the images array to an images sequence.
+    images = images_from_imagelist(self);
+
     if (argc == 1)
     {
         Image *img;
@@ -67,7 +75,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
 
     // If stack == true, stack rectangular images top-to-bottom,
     // otherwise left-to-right.
@@ -77,7 +85,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     new_image = AppendImages(images, stack, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -96,13 +104,13 @@ ImageList_average(VALUE self)
     ExceptionInfo exception;
 
     // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     new_image = AverageImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -124,13 +132,13 @@ ImageList_coalesce(VALUE self)
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     new_images = CoalesceImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -138,6 +146,69 @@ ImageList_coalesce(VALUE self)
 }
 
 
+/*
+    Method:     ImageList#composite_layers
+    Purpose:    Equivalent to convert's -layers composite option
+    Notes:      see mogrify.c
+*/
+VALUE
+ImageList_composite_layers(int argc, VALUE *argv, VALUE self)
+{
+#if defined(HAVE_COMPOSITELAYERS)
+    volatile VALUE source_images;
+    Image *dest, *source, *new_images;
+    RectangleInfo geometry;
+    CompositeOperator operator = OverCompositeOp;
+    ExceptionInfo exception;
+
+    switch (argc)
+    {
+        case 2:
+            VALUE_TO_ENUM(argv[1], operator, CompositeOperator);
+        case 1:
+            source_images = argv[0];
+            break;
+        default:
+            rb_raise(rb_eArgError, "wrong number of arguments (expected 1 or 2, got %d)", argc);
+            break;
+    }
+
+    // Convert ImageLists to image sequences.
+    dest = images_from_imagelist(self);
+    new_images = clone_imagelist(dest);
+    rm_split(dest);
+
+    source = images_from_imagelist(source_images);
+
+    SetGeometry(new_images,&geometry);
+    (void) ParseAbsoluteGeometry(new_images->geometry, &geometry);
+
+    geometry.width  = source->page.width != 0 ? source->page.width : source->columns;
+    geometry.height = source->page.height != 0 ? source->page.height : source->rows;
+    GravityAdjustGeometry(new_images->page.width  != 0 ? new_images->page.width  : new_images->columns
+                        , new_images->page.height != 0 ? new_images->page.height : new_images->rows
+                        , new_images->gravity, &geometry);
+
+    GetExceptionInfo(&exception);
+    CompositeLayers(new_images, operator, source, geometry.x, geometry.y, &exception);
+    rm_split(source);
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(&exception);
+
+    return rm_imagelist_from_images(new_images);
+
+#else
+
+    self = self;
+    argc = argc;
+    argv = argv;
+    rm_not_implemented();
+    return (VALUE)0;
+
+#endif
+}
+
+
 /*
     Method:     ImageList#deconstruct
     Purpose:    compares each image with the next in a sequence and returns
@@ -151,12 +222,12 @@ ImageList_deconstruct(VALUE self)
     Image *new_images, *images;
     ExceptionInfo exception;
 
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
     GetExceptionInfo(&exception);
     new_images = DeconstructImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -174,13 +245,13 @@ ImageList_display(VALUE self)
     Info *info;
     volatile VALUE info_obj;
 
-    // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
-
     // Create a new Info object to use with this call
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
 
+    // Convert the images array to an images sequence.
+    images = images_from_imagelist(self);
+
     (void) DisplayImages(info, images);
     rm_split(images);
     rm_check_image_exception(images, RetainOnError);
@@ -200,12 +271,57 @@ ImageList_flatten_images(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
     GetExceptionInfo(&exception);
+
+#if defined(HAVE_ENUM_FLATTENLAYER)
+    new_image = MergeImageLayers(images, FlattenLayer, &exception);
+#else
     new_image = FlattenImages(images, &exception);
+#endif
+
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_image);
+
+    return rm_image_new(new_image);
+}
+
+
+/*
+    Method:     ImageList#fx(expression[, channel...])
+*/
+VALUE
+ImageList_fx(int argc, VALUE *argv, VALUE self)
+{
+    Image *images, *new_image;
+    char *expression;
+    ChannelType channels;
+    ExceptionInfo exception;
+
+
+    channels = extract_channels(&argc, argv);
+
+    // There must be exactly 1 remaining argument.
+    if (argc == 0)
+    {
+        rb_raise(rb_eArgError, "wrong number of arguments (0 for 1 or more)");
+    }
+    else if (argc > 1)
+    {
+        raise_ChannelType_error(argv[argc-1]);
+    }
+
+    expression = StringValuePtr(argv[0]);
+
+    images = images_from_imagelist(self);
+    GetExceptionInfo(&exception);
+    new_image = FxImageChannel(images, channels, expression, &exception);
+    rm_split(images);
+    rm_check_exception(&exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -215,23 +331,34 @@ ImageList_flatten_images(VALUE self)
 
 
 /*
-    Method:     ImageList#map
+    Method:     ImageList#map(reference, dither=false)
     Purpose:    Call MapImages
     Returns:    a new ImageList with mapped images. @scene is set to self.scene
 */
 VALUE
-ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
+ImageList_map(int argc, VALUE *argv, VALUE self)
 {
     Image *images, *new_images = NULL;
     Image *map;
-    unsigned int dither;
-    volatile VALUE image, scene, new_imagelist;
+    unsigned int dither = MagickFalse;
+    volatile VALUE scene, new_imagelist, t;
     ExceptionInfo exception;
 
-    image = ImageList_cur_image(map_image);
-    Data_Get_Struct(image, Image, map);
+    switch (argc)
+    {
+        case 2:
+            dither = RTEST(argv[1]);
+        case 1:
+            t = ImageList_cur_image(argv[0]);
+            map = rm_check_destroyed(t);
+            break;
+        default:
+            rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+            break;
+    }
 
-    if (rm_imagelist_length(self) == 0)
+
+    if (imagelist_length(self) == 0L)
     {
         rb_raise(rb_eArgError, "no images in this image list");
     }
@@ -239,23 +366,22 @@ ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
     // Convert image array to image sequence, clone image sequence.
     GetExceptionInfo(&exception);
 
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
     new_images = CloneImageList(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
     // Call ImageMagick
-    dither = !(dither_arg == Qfalse || dither_arg == Qnil);
     (void) MapImages(new_images, map, dither);
     rm_check_image_exception(new_images, DestroyOnError);
 
     // Set @scene in new ImageList object to same value as in self.
     new_imagelist = rm_imagelist_from_images(new_images);
     scene = rb_iv_get(self, "@scene");
-    (void) rm_imagelist_scene_eq(new_imagelist, scene);
+    (void) imagelist_scene_eq(new_imagelist, scene);
 
     return new_imagelist;
 }
@@ -280,12 +406,12 @@ ImageList_montage(VALUE self)
     {
         // Run the block in the instance's context, allowing the app to modify the
         // object's attributes.
-        rb_obj_instance_eval(0, NULL, montage_obj);
+        (void) rb_obj_instance_eval(0, NULL, montage_obj);
     }
 
     Data_Get_Struct(montage_obj, Montage, montage);
 
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
 
     // If app specified a non-default composition operator, use it for all images.
     if (montage->compose != UndefinedCompositeOp)
@@ -303,7 +429,7 @@ ImageList_montage(VALUE self)
     new_images = MontageImages(images, montage->info, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -325,7 +451,7 @@ ImageList_morph(VALUE self, VALUE nimages)
     ExceptionInfo exception;
     long number_images;
 
-    if (rm_imagelist_length(self) < 1)
+    if (imagelist_length(self) < 1L)
     {
         rb_raise(rb_eArgError, "no images in this image list");
     }
@@ -337,12 +463,12 @@ ImageList_morph(VALUE self, VALUE nimages)
         rb_raise(rb_eArgError, "number of intervening images must be > 0");
     }
 
-    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
+    images = images_from_imagelist(self);
     new_images = MorphImages(images, (unsigned long)number_images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -360,12 +486,18 @@ ImageList_mosaic(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
+    images = images_from_imagelist(self);
+
+#if defined(HAVE_ENUM_MOSAICLAYER)
+    new_image = MergeImageLayers(images, MosaicLayer, &exception);
+#else
     new_image = MosaicImages(images, &exception);
+#endif
+
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -381,64 +513,110 @@ ImageList_mosaic(VALUE self)
 VALUE
 ImageList_optimize_layers(VALUE self, VALUE method)
 {
-#if defined(HAVE_COMPAREIMAGELAYERS)
-    Image *images, *new_images;
-    MagickLayerMethod mthd;
+    Image *images, *new_images, *new_images2;
+    LAYERMETHODTYPE mthd;
     ExceptionInfo exception;
 
-    images = rm_images_from_imagelist(self);
+    new_images2 = NULL;     // defeat "unused variable" message
+
     GetExceptionInfo(&exception);
+#if defined(HAVE_TYPE_IMAGELAYERMETHOD)
+    VALUE_TO_ENUM(method, mthd, ImageLayerMethod);
+#else
     VALUE_TO_ENUM(method, mthd, MagickLayerMethod);
+#endif
+    images = images_from_imagelist(self);
 
     switch (mthd)
     {
-#if defined(HAVE_COALESCELAYER)
-      case CoalesceLayer:
-        new_images = CoalesceImages(images, &exception);
-        break;
-      case DisposeLayer:
-        new_images = DisposeImages(images, &exception);
-        break;
+        case CoalesceLayer:
+            new_images = CoalesceImages(images, &exception);
+            break;
+        case DisposeLayer:
+            new_images = DisposeImages(images, &exception);
+            break;
+#if defined(HAVE_ENUM_OPTIMIZETRANSLAYER)
+        case OptimizeTransLayer:
+            new_images = clone_imagelist(images);
+            OptimizeImageTransparency(new_images, &exception);
+            break;
 #endif
-      case OptimizeLayer:
-        new_images = OptimizeImageLayers(images, &exception);
-        break;
-      case OptimizePlusLayer:
-        new_images = OptimizePlusImageLayers(images, &exception);
-        break;
-      default:
-        new_images = CompareImageLayers(images, mthd, &exception);
-        break;
+#if defined(HAVE_ENUM_REMOVEDUPSLAYER)
+        case RemoveDupsLayer:
+            new_images = clone_imagelist(images);
+            RemoveDuplicateLayers(&new_images, &exception);
+            break;
+#endif
+#if defined(HAVE_ENUM_REMOVEZEROLAYER)
+        case RemoveZeroLayer:
+            new_images = clone_imagelist(images);
+            RemoveZeroDelayLayers(&new_images, &exception);
+            break;
+#endif
+#if defined(HAVE_ENUM_COMPOSITELAYER)
+        case CompositeLayer:
+            rb_raise(rb_eNotImpError, "Magick::CompositeLayer is not yet supported.");
+            break;
+#endif
+#if defined(HAVE_ENUM_OPTIMIZEIMAGELAYER)
+            // In 6.3.4-ish, OptimizeImageLayer replaced OptimizeLayer
+        case OptimizeImageLayer:
+            new_images = OptimizeImageLayers(images, &exception);
+            break;
+            // and OptimizeLayer became a "General Purpose, GIF Animation Optimizer" (ref. mogrify.c)
+        case OptimizeLayer:
+            new_images = CoalesceImages(images, &exception);
+            rm_split(images);
+            rm_check_exception(&exception, new_images, DestroyOnError);
+            new_images2 = OptimizeImageLayers(new_images, &exception);
+            DestroyImageList(new_images);
+            rm_check_exception(&exception, new_images2, DestroyOnError);
+            new_images = new_images2;
+            OptimizeImageTransparency(new_images, &exception);
+            rm_check_exception(&exception, new_images, DestroyOnError);
+            // mogrify supports -dither here. We don't.
+            (void) MapImages(new_images, NULL, 0);
+            break;
+#else
+        case OptimizeLayer:
+            new_images = OptimizeImageLayers(images, &exception);
+            break;
+#endif
+        case OptimizePlusLayer:
+            new_images = OptimizePlusImageLayers(images, &exception);
+            break;
+        case CompareAnyLayer:
+        case CompareClearLayer:
+        case CompareOverlayLayer:
+            new_images = CompareImageLayers(images, mthd, &exception);
+            break;
+        default:
+            rb_raise(rb_eArgError, "undefined layer method");
+            break;
     }
 
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
     return rm_imagelist_from_images(new_images);
-
-#else
-    rm_not_implemented();
-    return (VALUE)0;
-#endif
 }
 
 
 /*
-    External:   rm_imagelist_new
+    Static:     ImageList_new
     Purpose:    create a new ImageList object with no images
     Notes:      this simply calls ImageList.new() in RMagick.rb
 */
-VALUE
-rm_imagelist_new()
+static VALUE
+ImageList_new(void)
 {
-    return rb_funcall(Class_ImageList, ID_new, 0);
+    return rb_funcall(Class_ImageList, rm_ID_new, 0);
 }
 
 
-
 /*
      Extern:   rm_imagelist_from_images
      Purpose:  Construct a new imagelist object from a list of images
@@ -447,51 +625,53 @@ rm_imagelist_new()
 VALUE
 rm_imagelist_from_images(Image *images)
 {
-     volatile VALUE new_imagelist;
-     Image *image;
+    volatile VALUE new_imagelist;
+    Image *image;
 
-     if (!images)
-     {
-         rb_bug("rm_imagelist_from_images called with NULL argument");
-     }
+    if (!images)
+    {
+        rb_bug("rm_imagelist_from_images called with NULL argument");
+    }
 
-     new_imagelist = rm_imagelist_new();
+    new_imagelist = ImageList_new();
 
-     while (images)
-     {
-          image = RemoveFirstImageFromList(&images);
-          rm_imagelist_push(new_imagelist, rm_image_new(image));
-     }
+    while (images)
+    {
+        image = RemoveFirstImageFromList(&images);
+        imagelist_push(new_imagelist, rm_image_new(image));
+    }
 
-     rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-     return new_imagelist;
+    (void) rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
+    return new_imagelist;
 }
 
 
 /*
-    Extern:     rm_images_from_imagelist
+    Extern:     images_from_imagelist
     Purpose:    Convert an array of Image *s to an ImageMagick scene
                 sequence (i.e. a doubly-linked list of Images)
     Returns:    a pointer to the head of the scene sequence list
 */
-Image *
-rm_images_from_imagelist(VALUE imagelist)
+static Image *
+images_from_imagelist(VALUE imagelist)
 {
     long x, len;
     Image *head = NULL;
+    volatile VALUE images, t;
 
-    Check_Type(imagelist, T_ARRAY);
-    len = rm_imagelist_length(imagelist);
+    len = imagelist_length(imagelist);
     if (len == 0)
     {
         rb_raise(rb_eArgError, "no images in this image list");
     }
 
+    images = rb_iv_get(imagelist, "@images");
     for (x = 0; x < len; x++)
     {
         Image *image;
 
-        Data_Get_Struct(rb_ary_entry(imagelist, x), Image, image);
+        t = rb_ary_entry(images, x);
+        image = rm_check_destroyed(t);
         AppendImageToList(&head, image);
     }
 
@@ -500,41 +680,66 @@ rm_images_from_imagelist(VALUE imagelist)
 
 
 /*
- *   Extern:   rm_imagelist_scene_eq(imagelist, scene)
+ *   Static:   imagelist_scene_eq(imagelist, scene)
  *   Purpose:  @scene attribute writer
 */
-VALUE
-rm_imagelist_scene_eq(VALUE imagelist, VALUE scene)
+static VALUE
+imagelist_scene_eq(VALUE imagelist, VALUE scene)
 {
-     rm_check_frozen(imagelist);
-     rb_iv_set(imagelist, "@scene", scene);
-     return scene;
+    rb_check_frozen(imagelist);
+    (void) rb_iv_set(imagelist, "@scene", scene);
+    return scene;
 }
 
 /*
-    External:   rm_imagelist_length
-    Purpose:    return the # of images in an imagelist
+    Static:    imagelist_length
+    Purpose:   return the # of images in an imagelist
 */
-int
-rm_imagelist_length(VALUE imagelist)
+static long
+imagelist_length(VALUE imagelist)
 {
-    volatile VALUE len;
-
-    len = rb_funcall(imagelist, ID_length, 0);
-    return FIX2INT(len);
+    volatile VALUE images = rb_iv_get(imagelist, "@images");
+    return RARRAY_LEN(images);
 }
 
 /*
-    External:   rm_imagelist_push
+    Static:     imagelist_push
     Purpose:    push an image onto the end of the imagelist
 */
-void
-rm_imagelist_push(VALUE imagelist, VALUE image)
+static void
+imagelist_push(VALUE imagelist, VALUE image)
 {
-    rm_check_frozen(imagelist);
-    (void) rb_funcall(imagelist, ID_push, 1, image);
+    rb_check_frozen(imagelist);
+    (void) rb_funcall(imagelist, rm_ID_push, 1, image);
 }
 
+
+/*
+ *  Static:     clone_imagelist
+ *  Purpose:    clone a list of images, handle errors
+ */
+static Image *
+clone_imagelist(Image *images)
+{
+    Image *new_imagelist = NULL, *image, *clone;
+    ExceptionInfo exception;
+
+    GetExceptionInfo(&exception);
+
+    image = GetFirstImageInList(images);
+    while (image)
+    {
+        clone = CloneImage(image, 0, 0, MagickTrue, &exception);
+        rm_check_exception(&exception, new_imagelist, DestroyOnError);
+        AppendImageToList(&new_imagelist, clone);
+        image = GetNextImageInList(image);
+    }
+
+    (void) DestroyExceptionInfo(&exception);
+    return new_imagelist;
+}
+
+
 /*
     Method:     ImageList#quantize(<number_colors<, colorspace<, dither<, tree_depth<, measure_error>>>>>)
                      defaults: 256, Magick::RGBColorspace, true, 0, false
@@ -556,15 +761,15 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     switch (argc)
     {
         case 5:
-            quantize_info.measure_error = RTEST(argv[4]);
+            quantize_info.measure_error = (MagickBooleanType) RTEST(argv[4]);
         case 4:
-            quantize_info.tree_depth = NUM2INT(argv[3]);
+            quantize_info.tree_depth = (unsigned long)NUM2INT(argv[3]);
         case 3:
-            quantize_info.dither = RTEST(argv[2]);
+            quantize_info.dither = (MagickBooleanType) RTEST(argv[2]);
         case 2:
             VALUE_TO_ENUM(argv[1], quantize_info.colorspace, ColorspaceType);
         case 1:
-            quantize_info.number_colors = NUM2INT(argv[0]);
+            quantize_info.number_colors = NUM2ULONG(argv[0]);
         case 0:
             break;
         default:
@@ -572,36 +777,36 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
             break;
     }
 
-    if (rm_imagelist_length(self) == 0)
+    if (imagelist_length(self) == 0L)
     {
         rb_raise(rb_eArgError, "no images in this image list");
     }
 
     // Convert image array to image sequence, clone image sequence.
     GetExceptionInfo(&exception);
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
     new_images = CloneImageList(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
 
-    QuantizeImages(&quantize_info, new_images);
+    (void) QuantizeImages(&quantize_info, new_images);
     rm_check_exception(&exception, new_images, DestroyOnError);
 
     // Create new ImageList object, convert mapped image sequence to images,
     // append to images array.
-    new_imagelist = rm_imagelist_new();
-    while ((new_image = ShiftImageList(&new_images)))
+    new_imagelist = ImageList_new();
+    while ((new_image = RemoveFirstImageFromList(&new_images)))
     {
-        rm_imagelist_push(new_imagelist, rm_image_new(new_image));
+        imagelist_push(new_imagelist, rm_image_new(new_image));
     }
 
     // Set @scene in new ImageList object to same value as in self.
     scene = rb_iv_get(self, "@scene");
-    rb_iv_set(new_imagelist, "@scene", scene);
+    (void) rb_iv_set(new_imagelist, "@scene", scene);
 
     return new_imagelist;
 }
@@ -627,11 +832,11 @@ ImageList_to_blob(VALUE self)
     Data_Get_Struct(info_obj, Info, info);
 
     // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
+    images = images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
-    (void) SetImageInfo(info, True, &exception);
-    CHECK_EXCEPTION()
+    (void) SetImageInfo(info, MagickTrue, &exception);
+    rm_check_exception(&exception, images, RetainOnError);
 
     if (*info->magick != '\0')
     {
@@ -645,19 +850,15 @@ ImageList_to_blob(VALUE self)
     // Unconditionally request multi-images support. The worst that
     // can happen is that there's only one image or the format
     // doesn't support multi-image files.
-    info->adjoin = True;
-#if defined(HAVE_IMAGESTOBLOB)
+    info->adjoin = MagickTrue;
     blob = ImagesToBlob(info, images, &length, &exception);
-#else
-    blob = ImageToBlob(info, images, &length, &exception);
-#endif
     if (exception.severity != UndefinedException)
     {
         magick_free((void*)blob);
     }
     rm_split(images);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
 
     if (length == 0 || !blob)
@@ -665,7 +866,7 @@ ImageList_to_blob(VALUE self)
         return Qnil;
     }
 
-    blob_str = rb_str_new(blob, length);
+    blob_str = rb_str_new(blob, (long)length);
     magick_free((void*)blob);
 
     return blob_str;
@@ -679,7 +880,7 @@ ImageList_to_blob(VALUE self)
 static VALUE file_arg_rescue(VALUE arg)
 {
     rb_raise(rb_eTypeError, "argument must be path name or open file (%s given)",
-            rb_class2name(CLASS_OF(arg)));
+             rb_class2name(CLASS_OF(arg)));
 }
 
 
@@ -699,17 +900,14 @@ ImageList_write(VALUE self, VALUE file)
     const MagickInfo *m;
     volatile VALUE info_obj;
     char *filename;
-    long filenameL;
-    int scene;
+    long filename_l;
+    unsigned long scene;
     ExceptionInfo exception;
 
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
 
 
-    // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
-
     if (TYPE(file) == T_FILE)
     {
         OpenFile *fptr;
@@ -724,13 +922,16 @@ ImageList_write(VALUE self, VALUE file)
         file = rb_rescue(rb_String, file, file_arg_rescue, file);
 
         // Copy the filename to the Info and to the Image.
-        filename = STRING_PTR_LEN(file, filenameL);
-        filenameL = min(filenameL, MaxTextExtent-1);
-        memcpy(info->filename, filename, (size_t)filenameL);
-        info->filename[filenameL] = '\0';
+        filename = rm_str2cstr(file, &filename_l);
+        filename_l = min(filename_l, MaxTextExtent-1);
+        memcpy(info->filename, filename, (size_t)filename_l);
+        info->filename[filename_l] = '\0';
         SetImageInfoFile(info, NULL);
     }
 
+    // Convert the images array to an images sequence.
+    images = images_from_imagelist(self);
+
     // Copy the filename into each images. Set a scene number to be used if
     // writing multiple files. (Ref: ImageMagick's utilities/convert.c
     for (scene = 0, img = images; img; img = GetNextImageInList(img))
@@ -740,17 +941,20 @@ ImageList_write(VALUE self, VALUE file)
     }
 
     GetExceptionInfo(&exception);
-    (void) SetImageInfo(info, True, &exception);
+    (void) SetImageInfo(info, MagickTrue, &exception);
+    rm_check_exception(&exception, images, RetainOnError);
+    (void) DestroyExceptionInfo(&exception);
 
     // Find out if the format supports multi-images files.
+    GetExceptionInfo(&exception);
     m = GetMagickInfo(info->magick, &exception);
-    CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    rm_check_exception(&exception, images, RetainOnError);
+    (void) DestroyExceptionInfo(&exception);
 
     // Tell WriteImage if we want a multi-images file.
-    if (rm_imagelist_length(self) > 1 && m->adjoin)
+    if (imagelist_length(self) > 1L && m->adjoin)
     {
-        info->adjoin = True;
+        info->adjoin = MagickTrue;
     }
 
     for (img = images; img; img = GetNextImageInList(img))
